from subprocess import check_output
from django.db import models
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.utils.functional import cached_property
from django.db.models.signals import post_save
from django.dispatch.dispatcher import receiver
from git.cmd import Git
from git.repo.base import Repo
from ..models import Commit
from ..managers import BranchManager
from ..decorators import start_new_thread


class Repository(models.Model):
    name = models.CharField(
        _('Name'),
        max_length=50,
        unique=True,
    )
    path = models.CharField(
        _('Path'),
        max_length=150,
        unique=True,
        blank=True,
        help_text=_('Basedir if blank'),
    )

    class Meta:
        verbose_name = _('Repository')
        verbose_name_plural = _('Repositories')

    @cached_property
    def repository(self):
        return Repo(path=self.path)

    @cached_property
    def git(self):
        return Git(self.path)

    def get_version(self):
        version = check_output(['python', f'{self.path}/setup.py', '--version'])
        return version.decode('utf-8')

    def clean(self):
        if not self.path:
            self.path = settings.BASE_DIR
        if not str(self.path).startswith('/'):
            self.path = f'{settings.BASE_DIR}/{self.path}'
        super().clean()

    def fetch(self):
        print('fetched', self)
        self.repository.remote().fetch()

    def refresh(self):
        disabled_branches = [branch.name for branch in self.branches.all()
                             if branch.enabled==False]
        for git_branch in self.repository.branches:
            if git_branch.name not in disabled_branches:
                branch = Branch.objects.get_or_create(
                    repository = self,
                    name=git_branch.name,
                )[0]
                commits = []
                for git_commit in self.repository.iter_commits(git_branch.name):
                    commit, updated = Commit.objects.update_or_create(
                        id=git_commit.hexsha,
                        repository=self,
                        defaults={
                            'message': git_commit.message,
                            'created_at': git_commit.committed_datetime,
                        }
                    )
                    commits.append(commit)
                branch.commits.set(commits)

    def __str__(self):
        return self.name


@receiver(post_save, sender=Repository, dispatch_uid="update_stock_count")
@start_new_thread
def refresh_on_save(instance, **kwargs):
    instance.refresh()


class Branch(models.Model):
    repository = models.ForeignKey(
        'Repository',
        related_name='branches',
        on_delete=models.CASCADE,
    )
    commits = models.ManyToManyField(
        'Commit',
        related_name='branches',
        blank=True,
    )
    name = models.CharField(
        _('Name'),
        max_length=50,
    )
    enabled = models.BooleanField(
        _('Enabled'),
        default=True,
    )

    objects = BranchManager()

    class Meta:
        unique_together = ['repository', 'name']
        verbose_name = _('Branch')
        verbose_name_plural = _('Branches')

    def __str__(self):
        return f'{self.repository.name}: {self.name}'



# Generated by Django 3.1.6 on 2021-02-17 00:26

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Branch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
            ],
            options={
                'verbose_name': 'Branch',
                'verbose_name_plural': 'Branches',
            },
        ),
        migrations.CreateModel(
            name='Human',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='Name')),
                ('email', models.EmailField(max_length=150, verbose_name='E-Mail')),
                ('master', models.BooleanField(default=False)),
                ('beta', models.BooleanField(default=False)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Mensch',
                'verbose_name_plural': 'Menschen',
            },
        ),
        migrations.CreateModel(
            name='Commit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hash', models.CharField(max_length=40, verbose_name='Id')),
                ('head', models.CharField(max_length=150, verbose_name='Titel')),
                ('body', models.TextField(blank=True, null=True, verbose_name='Beschreibung')),
                ('created_at', models.DateTimeField(blank=True, null=True, verbose_name='Übermittlungsdatum')),
                ('tag_name', models.CharField(blank=True, max_length=150, null=True, verbose_name='Tag-Name')),
                ('executed', models.BooleanField(default=False, help_text='Wurde dieser Comit bereits in einer E-Mail versendet?', verbose_name='Übermittelt')),
                ('branch', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='changelog.branch')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='changelog.human')),
            ],
            options={
                'ordering': ['-created_at'],
                'unique_together': {('hash', 'branch')},
            },
        ),
    ]



from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from telegram.bot import Bot


class TelegramBot(models.Model):
    id = models.CharField(
        _('Token'),
        max_length=250,
        help_text=_('eg. 123456789:QwErtzuIoPpjHgHlkhKGkjlHLhjLhlK-749'),
        primary_key=True
    )

    class Meta:
        verbose_name = _('Telegram bot')
        verbose_name_plural = _('Telegram bots')

    @cached_property
    def bot(self):
        return Bot(token=self.id)

    def __str__(self):
        return self.bot.name


class TelegramChat(models.Model):
    class Type(models.TextChoices):
        PRIVATE = 'private', _('Private')
        GROUP = 'group', _('Group')
        SUPERGROUP = 'supergroup', _('Supergroup')
        CHANNEL = 'channel', _('Channel')

    id = models.BigIntegerField(
        _('Chat ID'),
        primary_key=True,
    )
    telegram_bot = models.ForeignKey(
        'TelegramBot',
        on_delete=models.CASCADE,
    )
    type = models.CharField(max_length=10, choices=Type.choices)

    class Meta:
        verbose_name = _('Telegram chat')
        verbose_name_plural = _('Telegram chats')

    @cached_property
    def chat(self):
        return self.telegram_bot.bot.get_chat(self.id)

    def __str__(self):
        return self.chat.title



import os


def get_version():
    path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "VERSION")
    if os.path.isfile(path):
        with open(path) as f:
            return f.read().strip()
    return '0.0.0'


__version__ = get_version()



from django.contrib import admin
from django.shortcuts import redirect
from ..models import Repository, Branch, ReleaseProfile


class BranchInline(admin.TabularInline):
    model = Branch
    fields = ('name', 'enabled')
    list_fields = ('name', 'enabled')
    readonly_fields = ('name',)
    extra = 0


class ReleaseProfileInline(admin.StackedInline):
    model = ReleaseProfile
    extra = 0
    filter_horizontal = ('recipient_users', 'telegram_chats')


class RepositoryAdmin(admin.ModelAdmin):
    model = Repository
    inlines = [BranchInline, ReleaseProfileInline]

    def response_change(self, request, obj):
        if 'refresh' in request.POST:
            obj.fetch()
            obj.send()
            self.message_user(request, _('Refreshed all Repository objects.'))
            return redirect('admin:changelog_release_change', obj.id)
        return admin.ModelAdmin.response_change(self, request, obj)


admin.site.register(Repository, RepositoryAdmin)



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.shortcuts import redirect
from ..models import Commit, Tag, Branch, Release, ReleaseProfile


class BranchFilter(admin.SimpleListFilter):
    title = _('Branches')
    parameter_name = 'branches'

    def lookups(self, request, model_admin):
        for branch in Branch.objects.enabled().select_related('repository'):
            yield (branch.id, str(branch))

    def queryset(self, request, queryset):
        return queryset.filter(id__in=Commit.objects.enabled())


class TagInline(admin.TabularInline):
    model = Tag
    readonly_fields = ('commit', 'name', 'message')


class CommitAdmin(admin.ModelAdmin):
    change_form_template = 'admin/commit_change_form.html'
    model = Commit
    list_display = ('head', 'body', 'created_at', 'sent', 'hidden')
    list_filter = ('created_at', 'hidden', 'sent', BranchFilter,
                   'releases')
    readonly_fields = ('id', 'message')
    fields = readonly_fields
    search_fields = (
        'id', 'message', 'tags__name', 'tags__message', 'releases__version',
        'releases__email_recipient_users__email',
        'releases__messages__telegram_text',
        'releases__messages__email_subject',
        'releases__messages__email_text',
        'releases__messages__email_html',
    )
    actions = ['hide', 'unhide']

    def get_queryset(self, request):
        return Commit.objects.enabled()

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_actions(self, request):
        actions = {}
        for profile in ReleaseProfile.objects.all():
            func = self.__class__.add_to_release
            actions[str(profile)] = (
                func,
                str(profile),
                f'{func.short_description}: {profile}',
            )
        return actions

    @admin.display(description=_('Tags'))
    def get_tags_str(self, obj):
        return ', '.join([tag.name for tag in obj.tags.all()])

    @admin.action(description=_('Add to release'))
    def add_to_release(self, request, queryset):
        profile = ReleaseProfile.objects.get(name=request.POST['action'])
        repository = profile.repository
        commits = queryset.filter(branches__repository=repository)
        release = Release.from_profile(profile)
        release.commits.add(*commits)
        return redirect('admin:changelog_release_change', release.id)

    @admin.action(description=_('Hide selected'))
    def done(self, request, queryset):
        queryset.update(done=True)

    @admin.action(description=_('Unhide selected'))
    def undone(self, request, queryset):
        queryset.update(done=False)


admin.site.register(Commit, CommitAdmin)



# Generated by Django 3.2.7 on 2021-09-22 12:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('changelog', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='commit',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='commit',
            name='branch',
        ),
        migrations.RemoveField(
            model_name='commit',
            name='created_by',
        ),
        migrations.RemoveField(
            model_name='human',
            name='user',
        ),
        migrations.DeleteModel(
            name='Branch',
        ),
        migrations.DeleteModel(
            name='Commit',
        ),
        migrations.DeleteModel(
            name='Human',
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-23 12:43

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('changelog', '0002_delete_all_previous'),
    ]

    operations = [
        migrations.CreateModel(
            name='Commit',
            fields=[
                ('id', models.CharField(max_length=40, primary_key=True, serialize=False, verbose_name='SHA')),
                ('message', models.TextField(verbose_name='Message')),
                ('created_at', models.DateTimeField(verbose_name='Created at')),
                ('hidden', models.BooleanField(default=False, verbose_name='Hidden')),
                ('sent', models.BooleanField(default=False, verbose_name='Sent')),
                ('done', models.BooleanField(default=False, verbose_name='Done')),
            ],
            options={
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='Release',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('version', models.CharField(blank=True, editable=False, max_length=20)),
                ('text', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('sent_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Sent at')),
                ('commits', models.ManyToManyField(blank=True, related_name='releases', to='changelog.Commit')),
                ('email_recipient_users', models.ManyToManyField(blank=True, related_name='releases', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Release',
                'verbose_name_plural': 'Release',
            },
        ),
        migrations.CreateModel(
            name='Repository',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True, verbose_name='Name')),
                ('path', models.CharField(blank=True, help_text='Basedir if blank', max_length=150, unique=True, verbose_name='Path')),
            ],
            options={
                'verbose_name': 'Repository',
                'verbose_name_plural': 'Repositories',
            },
        ),
        migrations.CreateModel(
            name='TelegramBot',
            fields=[
                ('id', models.CharField(help_text='eg. 123456789:QwErtzuIoPpjHgHlkhKGkjlHLhjLhlK-749', max_length=250, primary_key=True, serialize=False, verbose_name='Token')),
            ],
            options={
                'verbose_name': 'Telegram bot',
                'verbose_name_plural': 'Telegram bots',
            },
        ),
        migrations.CreateModel(
            name='TelegramChat',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Chat ID')),
                ('type', models.CharField(choices=[('private', 'Private'), ('group', 'Group'), ('supergroup', 'Supergroup'), ('channel', 'Channel')], max_length=10)),
                ('telegram_bot', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='changelog.telegrambot')),
            ],
            options={
                'verbose_name': 'Telegram chat',
                'verbose_name_plural': 'Telegram chats',
            },
        ),
        migrations.CreateModel(
            name='ReleaseAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='', verbose_name='File')),
                ('release', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='changelog.release')),
            ],
            options={
                'verbose_name': 'Attachment',
                'verbose_name_plural': 'Attachments',
            },
        ),
        migrations.AddField(
            model_name='release',
            name='repository',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='changelog.repository'),
        ),
        migrations.AddField(
            model_name='release',
            name='telegram_chats',
            field=models.ManyToManyField(blank=True, related_name='releases', to='changelog.TelegramChat'),
        ),
        migrations.AddField(
            model_name='commit',
            name='repository',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='commits', to='changelog.repository'),
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='Name')),
                ('message', models.TextField(blank=True, null=True, verbose_name='Message')),
                ('commit', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tags', to='changelog.commit')),
            ],
            options={
                'verbose_name': 'Tag',
                'verbose_name_plural': 'Tags',
                'unique_together': {('commit', 'name')},
            },
        ),
        migrations.CreateModel(
            name='ReleaseProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True, verbose_name='Profile name')),
                ('text', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('recipient_users', models.ManyToManyField(blank=True, related_name='release_profiles', to=settings.AUTH_USER_MODEL)),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='changelog.repository')),
                ('telegram_chats', models.ManyToManyField(blank=True, related_name='release_profiles', to='changelog.TelegramChat')),
            ],
            options={
                'verbose_name': 'Release Profile',
                'verbose_name_plural': 'Release Profiles',
                'unique_together': {('name', 'repository')},
            },
        ),
        migrations.AlterUniqueTogether(
            name='release',
            unique_together={('repository', 'version')},
        ),
        migrations.CreateModel(
            name='Branch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='Name')),
                ('enabled', models.BooleanField(default=True, verbose_name='Enabled')),
                ('commits', models.ManyToManyField(blank=True, related_name='branches', to='changelog.Commit')),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='branches', to='changelog.repository')),
            ],
            options={
                'verbose_name': 'Branch',
                'verbose_name_plural': 'Branches',
                'unique_together': {('repository', 'name')},
            },
        ),
    ]



from django.core.management.base import BaseCommand
from changelog.utils import Changelog


class Command(BaseCommand):
    help = 'Refresh all Git entries in the Django Changelog Database'

    def handle(self, *args, **options):
        changelog = Changelog()
        changelog.fetch()
        changelog.refresh()



from django.contrib import admin
from changelog.models import TelegramBot, TelegramChat


class TelegramChatInline(admin.TabularInline):
    model = TelegramChat
    extra = 0
    

class TelegramBotAdmin(admin.ModelAdmin):
    model = TelegramBot
    inlines = [TelegramChatInline]


admin.site.register(TelegramBot, TelegramBotAdmin)



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.shortcuts import redirect
from django.conf import settings
from ..models import Release, ReleaseAttachment


class ReleaseAttachmentInline(admin.TabularInline):
    model = ReleaseAttachment
    extra = 0


class ReleaseAdmin(admin.ModelAdmin):
    change_form_template = 'admin/release_change_form.html'
    model = Release
    inlines = [ReleaseAttachmentInline]
    readonly_fields = ('sent_at', 'version',)
    filter_horizontal = ('commits', 'email_recipient_users', 'telegram_chats')
    fieldsets = (
        (None, {
            'fields': ('repository', 'version', 'text', 'sent_at',)
        }),
        (_('Linked objects'), {
            'classes': ('collapse',),
            'fields': filter_horizontal,
        }),
    )

    def has_change_permission(self, request, obj=None):
        return (obj and obj.sent_at == None) or settings.DEBUG

    def has_delete_permission(self, request, obj=None):
        return (obj and obj.sent_at == None) or settings.DEBUG

    def change_view(self, request, object_id, extra_context=None, *args,
                    **kwargs):
        extra_context = extra_context or {}
        obj = self.model.objects.get(id=object_id)
        extra_context['send_button_enabled'] = self.has_change_permission(
            request, obj
        )
        return super().change_view(
            request, object_id, extra_context=extra_context
        )

    def response_change(self, request, obj):
        if 'send' in request.POST:
            obj.save()
            result = obj.send()
            if result:
                self.message_user(request, _('E-Mail was sent.'))
            return redirect('admin:changelog_release_change', obj.id)
        return admin.ModelAdmin.response_change(self, request, obj)


admin.site.register(Release, ReleaseAdmin)



from django.db import models
from django.utils.translation import gettext_lazy as _
from ..managers import CommitManager
from django.utils.functional import cached_property


class Commit(models.Model):
    id = models.CharField(
        'SHA',
        max_length=40,
        primary_key=True,
    )
    repository = models.ForeignKey(
        'Repository',
        related_name='commits',
        on_delete=models.CASCADE,
    )
    message = models.TextField(
        _('Message'),
    )
    created_at = models.DateTimeField(
        _('Created at')
    )
    hidden = models.BooleanField(
        _('Hidden'),
        default=False,
    )
    sent = models.BooleanField(
        _('Sent'),
        default=False,
    )
    done = models.BooleanField(
        _('Done'),
        default=False,
    )

    objects = CommitManager()

    class Meta:
        ordering = ['-created_at']

    @cached_property
    def commit(self):
        for git_commit in self.repository.repository.iter_commits():
            if git_commit.hexsha == self.id:
                return git_commit

    @property
    def git(self):
        return self.repository.git

    @property
    def head(self):
        return self.message.split('\n', 1)[0]

    @property
    def body(self):
        return self.message.split('\n', 1)[1] if '\n' in self.message else None

    @property
    def stat(self):
        return self.git.show(self.id, stat=True)

    @property
    def details(self):
        return self.git.show(self.id)

    @property
    def files(self):
        return self.git.diff_tree(
            self.id, no_commit_id=True, name_only=True, r=True)

    @property
    def all_files(self):
        return self.git.ls_tree(self.id, name_only=True, r=True,)

    def __str__(self):
        return self.head


class Tag(models.Model):
    commit = models.ForeignKey(
        'Commit',
        on_delete=models.CASCADE,
        related_name='tags',
    )
    name = models.CharField(
        _('Name'),
        max_length=50,
    )
    message = models.TextField(
        _('Message'),
        blank=True, null=True,
    )

    class Meta:
        unique_together = ['commit', 'name']
        verbose_name = _('Tag')
        verbose_name_plural = _('Tags')

    def __str__(self):
        return self.name



from .commit import Commit, Tag
from .telegram import TelegramBot, TelegramChat
from .release import Release, ReleaseAttachment, ReleaseProfile
from .repository import Branch, Repository


__all__ = (
    'TelegramBot',
    'TelegramChat',
    'Release',
    'ReleaseAttachment',
    'ReleaseMessage',
    'ReleaseProfile',
    'EmailRecipient',
    'Branch',
    'Repository',
    'Commit',
    'Tag',
)



from django.db import models


class CommitManager(models.Manager):
    def enabled(self):
        return self.get_queryset().filter(branches__enabled=True).distinct()


class BranchManager(models.Manager):
    def enabled(self):
        return self.get_queryset().filter(enabled=True)



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
from django.template.loader import get_template
from django.conf import settings
from django.core.mail.message import EmailMultiAlternatives
from django.utils.timezone import now
from telegram.error import TimedOut
from ..decorators import start_new_thread

User = get_user_model()


class Release(models.Model):
    repository = models.ForeignKey(
        'Repository',
        on_delete=models.CASCADE,
    )
    version = models.CharField(
        max_length=20,
        editable=False,
        blank=True,
    )
    text = models.TextField(
        _('Text'),
        blank=True, null=True,
    )
    commits = models.ManyToManyField(
        'Commit',
        blank=True,
        related_name='releases',
    )
    telegram_chats = models.ManyToManyField(
        'TelegramChat',
        blank=True,
        related_name='releases',
    )
    email_recipient_users = models.ManyToManyField(
        User,
        blank=True,
        related_name='releases',
    )
    sent_at = models.DateTimeField(
        _('Sent at'),
        editable=False,
        blank=True, null=True,
    )

    class Meta:
        unique_together = ('repository', 'version')
        verbose_name = _('Release')
        verbose_name_plural = _('Release')

    @classmethod
    def from_profile(cls, profile):
        repository = profile.repository
        release, created = cls.objects.get_or_create(
            repository=repository,
            version=repository.get_version(),
            defaults={
                'text': profile.text
            }
        )
        if created:
            release.email_recipient_users.set(profile.recipient_users.all())
            release.telegram_chats.set(profile.telegram_chats.all())
        return release

    def clean(self):
        if not self.version:
            self.version = self.repository.get_version()
        super().clean()

    def get_context(self):
        return {'release': self, }

    def send_telegram_message(self, bot,  chat_id, text, **kwargs):
        try:
            bot.send_message(chat_id=chat_id, text=text, parse_mode='html')
        except TimedOut:
            pass

    @start_new_thread
    def send_telegram(self):
        ctx = self.get_context()
        for chat in self.telegram_chats.all():
            bot = chat.telegram_bot.bot
            first_ctx = {**ctx, 'text': self.text}
            rendered = get_template('changelog/telegram_first.html'
                                    ).render(first_ctx)
            self.send_telegram_message(bot, chat.id, rendered)
            for commit in self.commits.all():
                commit_ctx = {**ctx, 'commit': commit}
                rendered = get_template('changelog/telegram.html'
                                        ).render(commit_ctx)
                self.send_telegram_message(bot, chat.id, rendered)

    @start_new_thread
    def send_email(self):
        ctx = self.get_context()
        recipients = [f'{user.get_full_name()} <{user.email}>' for user
                      in self.email_recipient_users.all()]
        subject = get_template('changelog/subject.txt').render(ctx)
        subject = subject.replace("\n", "").replace("\r", "")
        body = get_template('changelog/email.txt').render(ctx)
        html = get_template('changelog/email.html').render(ctx)
        email = EmailMultiAlternatives(
                subject,
                body,
                from_email=settings.DEFAULT_FROM_EMAIL,
                to=recipients,
                # attachments=content_attachments,
            )
        email.attach_alternative(html, "text/html")
        email.send()

    def send(self):
        if self.sent_at and not settings.DEBUG:
            return
        self.commits.update(sent=True, hidden=True)
        self.send_telegram()
        self.send_email()
        self.sent_at = now()
        self.save()

    def __str__(self):
        result = self.repository.name
        if self.version:
            result += f' ver. {self.version}'
        return result


class ReleaseAttachment(models.Model):
    release = models.ForeignKey(
        'Release',
        related_name='attachments',
        on_delete=models.CASCADE,
    )
    file = models.FileField(
        _('File'),
    )

    class Meta:
        verbose_name = _('Attachment')
        verbose_name_plural = _('Attachments')


class ReleaseProfile(models.Model):
    repository = models.ForeignKey(
        'Repository',
        on_delete=models.CASCADE,
    )
    name = models.CharField(
        _('Profile name'),
        max_length=50,
        unique=True,
    )
    text = models.TextField(
        _('Text'),
        blank=True, null=True,
    )
    recipient_users = models.ManyToManyField(
        User,
        related_name='release_profiles',
        blank=True,
    )
    telegram_chats = models.ManyToManyField(
        'TelegramChat',
        related_name='release_profiles',
        blank=True,
    )

    class Meta:
        unique_together = ('name', 'repository')
        verbose_name = _('Release Profile')
        verbose_name_plural = _('Release Profiles')

    def __str__(self):
        return self.name



from threading import Thread


def start_new_thread(function):
    def decorator(*args, **kwargs):
        t = Thread(target=function, args=args, kwargs=kwargs)
        t.daemon = True
        t.start()
    return decorator



#!/usr/bin/env python
from setuptools import find_packages, setup
from src.changelog import __version__


install_requires = [
    'django>=3.2,<3.3',
]


tests_require = [
]


setup(
    name='django-git-changelog',
    version=__version__,
    author="Snake-Soft",
    author_email="info@snake-soft.com",
    description="List all git commits in the backend and create E-Mails with change informations",
    long_description=open('README.rst').read(),
    license='GPL3',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={
        'test': tests_require,
    },
)



from django.utils.functional import cached_property
from .models import Repository


class Changelog:
    @cached_property
    def repositories(self):
        return Repository.objects.all()

    def fetch(self):
        for repo in self.repositories:
            repo.fetch()

    def refresh(self):
        for repo in self.repositories:
            repo.refresh()
